/*****************************************************************************
		Newton's Method for Systems - Algorithm 10.1		
*******************************************************************************

To approximate the solution of the nonlinear system F(X)=0 given an initial
approximation X:

INPUT number n of equations and unknowns; initial approximation X=(x1,...,xn)t,
tolerance TOL; maximum number of iterations N; equations for F(X)=0 in
function f() and for J(X) in jacobian():

OUTPUT approximate solution X=(x1,...,xn)t or a message that the number
of iterations was exceeded.

*******************************************************************************
*	Written by:  Harold A. Toomey, BYU, Summer 1989, v4.11		      *
******************************************************************************/

#include <stdio.h>		/* for scanf() and printf() */
#include <math.h>		/* for fabs() */
#include "naa_util.h"		/* Numerical Analysis Algorithms Utilities. */
#include "gaussj.h"		/* Guass-Jordan method (inverts a matrix) */
#define	PI	3.14159265358979323846264338327950288419716939937510582097

int n;				/* dimension of n x n matrix */


main()
{
  double **J, **JINV, **F, **Y, *X, TOL;
  double f(), jacobian(), infinite_norm();
  int i, j, k, N;

  get_title();

  printf("Newton's Method for Systems - Algorithm 10.1\n\n");
  fprintf(stderr, "Newton's Method for Systems - Algorithm 10.1\n\n");

  fprintf(stderr, "Enter maximum number of iterations, N: ");
  scanf("%d", &N);
  printf("Maximum number of iterations: %d\n", N);

  fprintf(stderr, "Enter Tolerance TOL (1.0e-5)?: ");
  scanf("%lf", &TOL);
  printf("Tolerance: %lg\n", TOL);

  fprintf(stderr, "Enter number of equations, n: ");
  scanf("%d", &n);
  if (n <= 0)
    naaerror("n must be > 0.");	/* ERROR. */
  printf("System has %d equations and %d unknowns.\n\n", n, n);

  J = dmatrix(1,n,1,n);		/* Jacobian Matrix */
  JINV = dmatrix(1,n,1,n);	/* Inverse of J */
  F = dmatrix(1,n,1,1);		/* Function Matrix */
  Y = dmatrix(1,n,1,1);		/* for -Jinv[][]*F[][] */
  X = dvector(1,n);		/* solution vector */

  fprintf(stderr, "Enter initial approximations to X:\n");
  for (i=1;i<=n;i++) {
    fprintf(stderr, "X[%d] =  ", i);
    scanf("%lf", &X[i]);
  }

  /*-------------------------------------------------------------------------*/
  printf("\n k\t");		/* print out table header */
  for (i=1;i<=n;i++)
    printf("  X[%d]\t\t", i);
  printf("\n---------");
  for (i=1;i<=n;i++)
    printf("---------------");
  printf("\n");
  /*-------------------------------------------------------------------------*/

  printf("% d\t", 0);
  for (i=1;i<=n;i++)
    printf("% .9lf\t", X[i]);
  printf("\n");

  /* STEP #1 */
  k = 1;

  /* STEP #2 */
  while (k <= N) {		/* Do Steps 3-7. */

    /* STEP #3 */
    for (i=1;i<=n;i++) {	/* Calculate F(X) and J(X), where J(X)ij = */
      F[i][1] = f(i, X);	/* (dfi(X)/dxj) for 1 <= i,j <= n. */
      for (j=1;j<=n;j++)
        J[i][j] = jacobian(i, j, X);
    }

    /* STEP #4 */
    /* Solve the n x n linear system J(X)*Y = -F(X). */
    for (i=1;i<=n;i++) {
      Y[i][1] = -F[i][1];	/* Y = -F */
      for (j=1;j<=n;j++)
        JINV[i][j] = J[i][j];	/* Copy J into JINV before passing to gaussj */
    }
    gaussj(JINV,n,Y,1);		/* Solves for Y of J*Y = -F. */

    /* STEP #5 */
    for (i=1;i<=n;i++)		/* set X = X + Y. */
      X[i] += Y[i][1];

    /* STEP #6 */
    printf("% d\t", k);		/* print out solution to kth iteration. */
    for (i=1;i<=n;i++)
      printf("% .9lf\t", X[i]);
    printf("\n");
    if (infinite_norm(Y) < TOL)	/* check for requested accuracy. */
      exit (1);			/* Procedure completed successfully. */

    /* STEP #7 */
    k++;

  }

  /* STEP #8 */			/* Procedure completed unsuccessfully */
  printf("\nMaximum number of iterations (%d) exceeded.\n", N);

}  /* STOP */

/*****************************************************************************/
double infinite_norm(X)	/* Computes the Infinite Norm for vectors. */
double **X;
{
  double max = 0.0;
  int i;

  for (i=1;i<=n;i++)
    if (fabs(X[i][1]) > max)
      max = fabs(X[i][1]);
  return (max);
}
/***************************************************************************/
/*****************************************************************************/
double f(i, X)		/* Function Matrix */
int i;			/* selects function number */
double *X;
{
  switch (i) {
    case 1: return (3.0*X[1] - cos(X[2]*X[3]) - 0.5);
    case 2: return (X[1]*X[1] - 81.0*(X[2]+0.1)*(X[2]+0.1) + sin(X[3]) + 1.06);
    case 3: return (exp(-X[1]*X[2]) + 20.0*X[3] + (10.0*PI - 3.0)/3.0);
    default: printf("Oops!  You forgot to modify f().\n");
             naaerror("Oops!  You forgot to modify f().");
  }
}

/*****************************************************************************/
double jacobian(i, j, X) /* Jacobian Matrix */
int i, j;		/* selects function number */
double *X;
{
  switch (j) {
    case 1:
      switch (i) {
        case 1: return (3.0);
        case 2: return (2.0*X[1]);
        case 3: return (-X[2]*exp(-X[1]*X[2]));
        default: printf("Oops!  You forgot to modify jacobian().\n");
                 naaerror("Oops!  You forgot to modify jacobian().");
      }

    case 2:
      switch (i) {
        case 1: return (X[3]*sin(X[2]*X[3]));
        case 2: return (-162.0*(X[2]+0.1));
        case 3: return (-X[1]*exp(-X[1]*X[2]));
      }

    case 3:
      switch (i) {
        case 1: return (X[2]*sin(X[2]*X[3]));
        case 2: return (cos(X[3]));
        case 3: return (20.0);
      }
  }
}
/*****************************************************************************/
