/******************************************************************************
	"Numerical Analysis Algorithms in C" Utilities I v4.11
******************************************************************************/

/*
  Note: Many of these utilities were borrowed from the book "Numerical
        Recipes in C" after being improved and modified.
*/

/* Must have '#include <stdio.h>' before '#include "naa_util.h"' in all main()
   programs.  For separate compilation, remove the comment symbols below. */
/*#include <stdio.h>*/		/* For stderr definition. */

/* #include <stdlib.h>		/* For calloc(). */


/* NEW STANDARD PROTOTYPING */
/*
void nrerror(char []);
void naaerror(char []);
double **dmatrix(int, int, int, int);
double *dvector(int, int);
float **matrix(int, int, int, int);
float *vector(int, int);
int *ivector(int, int);
void free_dmatrix(double **, int, int, int, int);
void free_matrix(float **, int, int, int, int);
void free_dvector(double *, int, int);
void free_vector(float *, int, int);
void free_ivector(int *, int, int);
void get_title(void);
*/


/* OLD STANDARD PROTOTYPING */
void nrerror();
void naaerror();
double **dmatrix();
double *dvector();
float **matrix();
float *vector();
int *ivector();
void free_dmatrix();
void free_matrix();
void free_dvector();
void free_vector();
void free_ivector();
void get_title();

/*****************************************************************************/
void naaerror(error_text)
char error_text[];
/* Numerical Analysis Algorithms standard error handler. */
{
  fprintf(stderr,"\nNumerical Analysis Algorithms run-time error...\n");
  fprintf(stderr,"%s\n...now exiting to system...\n",error_text);
  exit(1);
}

/*****************************************************************************/
void nrerror(error_text)
char error_text[];
/* Numerical Recipes standard error handler. */
{
  fprintf(stderr,"\nNumerical Recipes run-time error...\n");
  fprintf(stderr,"%s\n...now exiting to system...\n",error_text);
  exit(1);
}

/*****************************************************************************/
double **dmatrix(a,b,c,d)
int a,b,c,d;
/* allocates a double matrix with range [a..b][c..d]. */
{
  int i;
  double **m;

  /* allocate pointers to rows */
  m=(double **) calloc((unsigned) (b-a+1), sizeof(double*));
  if (!m) nrerror("allocation failure 1 in dmatrix()");
  m -= a;

  for (i=a;i<=b;i++) {	/* allocate rows and set pointers to them */
    m[i]=(double *) calloc((unsigned) (d-c+1), sizeof(double));
    if (!m[i]) nrerror("allocation failure 2 in dmatrix()");
    m[i] -= c;
  }

  return (m);		/* return pointer to array of pointers to rows */
}

/*****************************************************************************/
double *dvector(a,b)
int a,b;
/* allocates a double vector with range [a..b]. */
{
  double *v;

  v=(double *)calloc((unsigned) (b-a+1), sizeof(double));
  if (!v) nrerror("allocation failure in dvector()");
  return (v-a);
}

/*****************************************************************************/
float **matrix(a,b,c,d)
int a,b,c,d;
/* allocates a float matrix with range [a..b][c..d]. */
{
  int i;
  float **m;

  /* allocate pointers to rows */
  m=(float **) calloc((unsigned) (b-a+1), sizeof(float*));
  if (!m) nrerror("allocation failure 1 in dmatrix()");
  m -= a;

  for (i=a;i<=b;i++) {	/* allocate rows and set pointers to them */
    m[i]=(float *) calloc((unsigned) (d-c+1), sizeof(float));
    if (!m[i]) nrerror("allocation failure 2 in dmatrix()");
    m[i] -= c;
  }

  return (m);		/* return pointer to array of pointers to rows */
}

/*****************************************************************************/
float *vector(a,b)
int a,b;
/* allocates a float vector with range [a..b]. */
{
  float *v;

  v=(float *)calloc((unsigned) (b-a+1), sizeof(float));
  if (!v) nrerror("allocation failure in dvector()");
  return (v-a);
}

/*****************************************************************************/
int *ivector(a,b)
int a,b;
/* allocates an integer vector with range [a..b]. */
{
  int *v;

  v=(int *) calloc((unsigned) (b-a+1), sizeof(int));
  if (!v) nrerror("allocation failure in ivector()");
  return (v-a);
}

/*****************************************************************************/
void free_dmatrix(m,a,b,c,d)
double **m;
int a,b,c,d;		/* (variable d is never used.) */
/* frees a matrix allocated with dmatrix(). */
{
  int i;

  for(i=b;i>=a;i--)
    free((char*) (m[i]+c));
  free((char*) (m+a));
}

/*****************************************************************************/
void free_matrix(m,a,b,c,d)
float **m;
int a,b,c,d;		/* (variable d is never used.) */
/* frees a matrix allocated with matrix(). */
{
  int i;

  for(i=b;i>=a;i--)
    free((char*) (m[i]+c));
  free((char*) (m+a));
}

/*****************************************************************************/
void free_dvector(v,a,b)
double *v;
int a,b;		/* (variable b is never used.) */
/* frees a double vector allocated by dvector(). */
{
  free((char*) (v+a));
}

/*****************************************************************************/
void free_vector(v,a,b)
float *v;
int a,b;		/* (variable b is never used.) */
/* frees a float vector allocated by vector(). */
{
  free((char*) (v+a));
}

/*****************************************************************************/
void free_ivector(v,a,b)
int *v,a,b;		/* (variable b is never used.) */
/* frees an int vector allocated by ivector(). */
{
  free((char*) (v+a));
}

/*****************************************************************************/
#define	TITLELEN	133	/* Maximum title length. */

void get_title()
/* Prompts for a title and prints it, unless it is only "\n". */
{
  char title[TITLELEN];
  int i=0;

  fprintf(stderr,
     "Enter a title [i.e. - Set 2.1,  Problem 2 a),  p.36 ].\n----> ");
  while (((title[i++] = getchar()) != '\n') && (i < TITLELEN))
    ;
  title[i] = '\0';
  if (i > 1)  printf("%s\n", title);
}

/******************************************************************************
*	Written by:  Harold A. Toomey, BYU, Summer 1989, v4.11		      *
******************************************************************************/
